@@ -8,7 +8,7 @@ use SVK::I18N;
 use SVK::Editor::Delay;
 use SVK::Command::Log;
 use SVK::Merge;
-use SVK::Util qw( get_buffer_from_editor find_svm_source resolve_svm_source );
+use SVK::Util qw( get_buffer_from_editor find_svm_source resolve_svm_source traverse_history );
 
 sub options {
     ($_[0]->SUPER::options,
@@ -20,6 +20,7 @@ sub options {
      'I|incremental'	=> 'incremental',
      'no-ticket'	=> 'no_ticket',
      'r|revision=s'	=> 'revspec',
+     'c|change=s',	=> 'chgspec',
      't|to'             => 'to',
      'f|from'           => 'from',
      's|sync'           => 'sync');
@@ -30,12 +31,10 @@ sub parse_arg {
     return if $#arg > 1;
 
     if (!$self->{to} && !$self->{from}) {
-
-        if (scalar (@arg) == 0) {
-            return;
-        }
-
-        return ($self->arg_depotpath ($arg[0]), $self->arg_co_maybe ($arg[1] || ''));
+        return if scalar (@arg) == 0;
+	my ($src, $dst) = ($self->arg_depotpath ($arg[0]), $self->arg_co_maybe ($arg[1] || ''));
+	die loc("Can't merge across depots.\n") unless $src->same_repos ($dst);
+        return ($src, $dst);
     }
 
     if (scalar (@arg) == 2) {
@@ -75,22 +74,19 @@ sub get_commit_message {
     my ($self, $log) = @_;
     return if $self->{check_only} || $self->{incremental};
     $self->{message} = defined $self->{message} ?
-	join ("\n", $self->{message}, $log, '')
+	join ("\n", grep {length $_} ($self->{message}, $log))
 	    : $self->SUPER::get_commit_message ($log);
 }
 
 sub run {
     my ($self, $src, $dst) = @_;
     my $merge;
-    die loc("repos paths mismatch") unless $src->same_repos ($dst);
     my $repos = $src->{repos};
     my $fs = $repos->fs;
     my $yrev = $fs->youngest_rev;
 
     if ($self->{sync}) {
-        require SVK::Command::Sync;
-        my $sync = SVK::Command::Sync->new;
-        %$sync = (%$self, %$sync);
+        my $sync = $self->command ('sync');
 	my (undef, $m) = resolve_svm_source($repos, find_svm_source($repos, $src->{path}));
         if ($m->{target_path}) {
             $sync->run($self->arg_depotpath('/' . $src->depotname .  $m->{target_path}));
@@ -105,6 +101,7 @@ sub run {
     if ($self->{auto}) {
 	die loc("No need to track rename for smerge\n")
 	    if $self->{track_rename};
+	++$self->{no_ticket} if $self->{patch};
 	# XXX: these should come from parse_arg
 	$src->normalize; $dst->normalize;
 	$merge = SVK::Merge->auto (%$self, repos => $repos, target => '',
@@ -114,13 +111,12 @@ sub run {
     }
     else {
 	die loc("Incremental merge not supported\n") if $self->{incremental};
-	die loc("Revision required\n") unless $self->{revspec};
-	my ($baserev, $torev) = $self->{revspec} =~ m/^(\d+):(\d+)$/
-	    or die loc("Revision must be N:M\n");
-	$src->{revision} = $torev;
+	my @revlist = $self->parse_revlist;
+	die "multi-merge not yet" if $#revlist > 0;
+	my ($baserev, $torev) = @{$revlist[0]};
 	$merge = SVK::Merge->new
-	    (%$self, repos => $repos, src => $src, dst => $dst,
-	     base => $src->new (revision => $baserev), target => '');
+	    (%$self, repos => $repos, src => $src->new (revision => $torev),
+	     dst => $dst, base => $src->new (revision => $baserev), target => '');
     }
 
     $self->get_commit_message ($self->{log} ? $merge->log(1) : '')
@@ -134,18 +130,22 @@ sub run {
 	    if $self->{no_ticket};
 	print loc ("-m ignored in incremental merge\n") if $self->{message};
 	my @rev;
-	my $hist = $src->root->node_history ($src->{path});
+
+        traverse_history (
+            root        => $src->root,
+            path        => $src->{path},
+            cross       => 0,
+            callback    => sub {
+                my $rev = $_[1];
+                return 0 if $rev <= $merge->{fromrev}; # last
+                unshift @rev, $rev;
+                return 1;
+            },
+        );
+
 	my $spool = SVN::Pool->new_default;
-	while ($hist = $hist->prev (0)) {
-	    my $rev = ($hist->location)[1];
-	    last if $rev <= $merge->{fromrev};
-	    unshift @rev, $rev;
-	    $spool->clear;
-	}
 	for (@rev) {
-	    $src->{revision} = $_;
-	    $merge = SVK::Merge->auto (%$self, repos => $repos, ticket => 1,
-				       src => $src, dst => $dst);
+	    $merge = SVK::Merge->auto (%$merge, src => $src->new (revision => $_));
 	    print '===> '.$merge->info;
 	    $self->{message} = $merge->log (1);
 	    last if $merge->run ($self->get_editor ($dst));
@@ -157,7 +157,7 @@ sub run {
     else {
 	print loc("Incremental merge not guaranteed even if check is successful\n")
 	    if $self->{incremental};
-	$merge->run ($self->get_editor ($dst));
+	$merge->run ($self->get_editor ($dst, undef, $self->{auto} ? $src : undef));
     }
     return;
 }
